/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WFPELSFUNCTION_H
#define WFPELSFUNCTION_H

#include "dart.h"
#include "wLinesearch.h"
#include "wNewton.h"

namespace dart
{

/**
 * @brief Line search function for Full Potential Equation
 * @authors Adrien Crovato
 */
class DART_API FpeLSFunction : public tbox::LSFunction
{
private:
    Newton &solver;                    ///< solver object (must be of Newton type)
    std::vector<double> &u;            ///< solution
    std::vector<double> u0;            ///< initial solution
    Eigen::Map<Eigen::VectorXd> du;    ///< change in solution
    Eigen::Map<Eigen::VectorXd> &fRes; ///< residual vector

public:
    FpeLSFunction(Newton &_solver, std::vector<double> &_u, std::vector<double> &_u0, Eigen::Map<Eigen::VectorXd> &_du, Eigen::Map<Eigen::VectorXd> &_fRes);
    void update(std::vector<double> &_u0, Eigen::Map<Eigen::VectorXd> &_du);
    virtual double eval(double alpha) override;

#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif
};

} // namespace dart

#endif //WFPELSFUNCTION_H
