#include "wDartCoupler.h"
#include "../../dart/src/wNewton.h"
#include "../../dart/src/wBlowing.h"
#include "wViscSolver.h"

using namespace vii;

DartCoupler::DartCoupler(const Newton *_invSolver, ViscSolver *_viscSolver, const std::vector<*Blowing> _blw, unsigned long _nCouplIter = 150, double _tol = 1e-4)
{
    invSolver = _invSolver;
    viscSolver = _viscSolver;
    blw = _blw;

    maxCouplIter = _nCouplIter;
    tol = _tol;

    Initialize();
}
DartCoupler::~DartCoupler()

void DartCoupler::Run(){
    double cdPrev = 0.;
    double error = 1.;

    unsigned long nIters;

    while (nIters < maxCouplIter){

        invSolver.run();

        ImposeInviscidBC();

        viscSolver.Run();

        error = (viscSolver.Cdt - cdPrev) / viscSolver.Cdt;

        if (error < tol){
            break;
        }
        else{
            continue;
        }

        cdPrev = viscSolver.Cdt;

        ImposeBlowing();

        nIters++;

    }

    else{
        std::cout << "Maximum number of " << maxCouplIter << " reached." << std::endl;
    }
}
void DartCoupler::ImposeInviscidBC(){

    for (int iReg=0; iReg < blw.size(); ++iReg){

        for (auto iNode : blw[iReg]->nodes){
            invSolver->U(iNode->row)
        }
    }
}

void DartCoupler::ImposeBlowing()

void DartCoupler::Initialize(){

    /* Create connectivity list */

    std::vector<std::vector<unsigned int>,2> connectListNodes;
    std::vector<std::vector<unsigned int>,2> connectListElems;

    for (size_t iReg=0; iReg < blw.size(); ++iReg)
    {
        connectListNodes(iReg).resize(blw(iReg)->boundary->nodes.size(), 0);
        connectListElems(iReg).resize(blw(iReg)->boudnary->tag->elems.size(), 0);
    }


    /* Find stagnation point */
    int idxStag = 


    N1 = np.zeros(self.nN, dtype=int) # Node number
    connectListNodes = np.zeros(self.nN, dtype=int) # Index in boundary.nodes
    connectListElems = np.zeros(self.nE, dtype=int) # Index in boundary.elems
    data = np.zeros((self.boundary.nodes.size(), 10))
    i = 0
    for n in self.boundary.nodes:
        data[i,0] = n.no
        data[i,1] = n.pos[0]
        data[i,2] = n.pos[1]
        data[i,3] = n.pos[2]
        data[i,4] = self.v[i,0]
        data[i,5] = self.v[i,1]
        data[i,6] = self.Me[i]
        data[i,7] = self.rhoe[i]
        data[i,8] = self.deltaStar[i]
        data[i,9] = self.xx[i]
        i += 1
    # Table containing the element and its nodes
    eData = np.zeros((self.nE,3), dtype=int)
    for i in range(0, self.nE):
        eData[i,0] = self.boundary.tag.elems[i].no
        eData[i,1] = self.boundary.tag.elems[i].nodes[0].no
        eData[i,2] = self.boundary.tag.elems[i].nodes[1].no
    # Find the stagnation point
    if couplIter == 0:
        self.idxStag = np.argmin(np.sqrt(data[:,4]**2+data[:,5]**2))
        idxStag = self.idxStag
    else:
        idxStag = self.idxStag
    #idxStag = np.argmin(np.sqrt(data[:,4]**2+data[:,5]**2))
    globStag = int(data[idxStag,0]) # position of the stagnation point in boundary.nodes
    # Find TE nodes (assuming suction side element is above pressure side element in the y direction)
    idxTE = np.where(data[:,1] == np.max(data[:,1]))[0] # TE nodes
    idxTEe0 = np.where(np.logical_or(eData[:,1] == data[idxTE[0],0], eData[:,2] == data[idxTE[0],0]))[0] # TE element 1
    idxTEe1 = np.where(np.logical_or(eData[:,1] == data[idxTE[1],0], eData[:,2] == data[idxTE[1],0]))[0] # TE element 2
    ye0 = 0.5 * (data[np.where(data[:,0] == eData[idxTEe0[0],1])[0],2] + data[np.where(data[:,0] == eData[idxTEe0[0],2])[0],2]) # y coordinates of TE element 1 CG
    ye1 = 0.5 * (data[np.where(data[:,0] == eData[idxTEe1[0],1])[0],2] + data[np.where(data[:,0] == eData[idxTEe1[0],2])[0],2]) # y coordinates of TE element 2 CG
    if ye0 - ye1 > 0: # element 1 containing TE node 1 is above element 2
        upperGlobTE = int(data[idxTE[0],0])
        lowerGlobTE = int(data[idxTE[1],0])
    else:
            upperGlobTE = int(data[idxTE[1],0])
            lowerGlobTE = int(data[idxTE[0],0])
    # Connectivity
    connectListElems[0] = np.where(eData[:,1] == globStag)[0]
    N1[0] = eData[connectListElems[0],1] # number of the stag node elems.nodes -> globStag
    connectListNodes[0] = np.where(data[:,0] == N1[0])[0]
    i = 1
    upperTE = 0
    lowerTE = 0
    # Sort the suction part
    while upperTE == 0:
        N1[i] = eData[connectListElems[i-1],2] # Second node of the element
        connectListElems[i] = np.where(eData[:,1] == N1[i])[0] # # Index of the first node of the next element in elems.nodes
        connectListNodes[i] = np.where(data[:,0] == N1[i])[0] # Index of the node in boundary.nodes
        if eData[connectListElems[i],2] == upperGlobTE:
            upperTE = 1
        i += 1
    # Sort the pressure side
    connectListElems[i] = np.where(eData[:,2] == globStag)[0]
    connectListNodes[i] = np.where(data[:,0] == upperGlobTE)[0]
    N1[i] = eData[connectListElems[i],2]
    while lowerTE == 0:
        N1[i+1]  = eData[connectListElems[i],1] # First node of the element
        connectListElems[i+1] = np.where(eData[:,2] == N1[i+1])[0] # # Index of the second node of the next element in elems.nodes
        connectListNodes[i+1] = np.where(data[:,0] == N1[i+1])[0] # Index of the node in boundary.nodes
        if eData[connectListElems[i+1],1] == lowerGlobTE:
            lowerTE = 1
        i += 1
    connectListNodes[i+1] = np.where(data[:,0] == lowerGlobTE)[0]
    data[:,0] = data[connectListNodes,0]
    data[:,1] = data[connectListNodes,1]
    data[:,2] = data[connectListNodes,2]
    data[:,3] = data[connectListNodes,3]
    data[:,4] = data[connectListNodes,4]
    data[:,5] = data[connectListNodes,5]
    data[:,6] = data[connectListNodes,6]
    data[:,7] = data[connectListNodes,7]
    data[:,8] = data[connectListNodes,8]
    data[:,9] = data[connectListNodes,9]
    # Separated upper and lower part
    data = np.delete(data,0,1)
    uData = data[0:np.argmax(data[:,0])+1]
    lData = data[np.argmax(data[:,0])+1:None]
    lData = np.insert(lData, 0, uData[0,:], axis = 0) #double the stagnation point
    return connectListNodes, connectListElems,[uData, lData]
}
